/*
 * 
 * Copyright (C) 2001 by Dom Lachowicz
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <Magick++.h>

#include "ut_types.h"
#include "xap_Module.h"
#include "ie_impGraphic.h"
#include "fg_GraphicRaster.h"

#ifdef _WIN32  //Braindead LIBC library requires this
int main()
{
	return 0;
}
#endif

// -----------------------------------------------------------------------
// -----------------------------------------------------------------------

/*!
 * This class will import any graphic type supported
 * by ImageMagick into a PNG buffer for AbiWord's use
 */
class ABI_EXPORT IE_MagickGraphic : public IE_ImpGraphic
{

public:

  IE_MagickGraphic ()
  {
  }

  virtual ~IE_MagickGraphic ()
  {
  } 
  
  /*!
   * Convert the incoming byte-buffer
   * into a PNG-based FG_Graphic type
   */
  virtual UT_Error  importGraphic(UT_ByteBuf* pBB, 
				  FG_Graphic ** ppfg)
  {
    UT_Error err = UT_OK;
    UT_ByteBuf * pBBpng = 0;	
    FG_GraphicRaster * pFGR = 0;
    
    *ppfg = 0;
    
    err = convertGraphic(pBB,&pBBpng);
    if (err != UT_OK) {
      return err;
    }

    pFGR = new FG_GraphicRaster();
    if(pFGR == 0) {
      err = UT_IE_NOMEMORY;
    }
    else if(!pFGR->setRaster_PNG(pBBpng)) {
      DELETEP(pFGR);
      err = UT_IE_FAKETYPE;
    }
    else {
      *ppfg = (FG_Graphic *) pFGR;
    }

    return err;
  }

  /*!
   * Convert the incoming byte-buffer into a PNG byte-buffer
   */
  virtual UT_Error convertGraphic(UT_ByteBuf* pBB, 
				  UT_ByteBuf** ppBB)
  {
    try
      {
	Magick::Blob  inBlob ( pBB->getPointer (0), pBB->getLength () );
	Magick::Image img ( inBlob );
	
	Magick::Blob outBlob;
	img.write ( &outBlob, "png" );
	
	// write out the IM::Blob to our Byte-Buffer
	UT_ByteBuf * buf = new UT_ByteBuf (outBlob.length ());
	buf->append ( (const UT_Byte*)outBlob.data (), outBlob.length () );
	*ppBB = buf;
	
	return UT_OK;
      }
    catch (...)
      {
	return UT_ERROR;
      }
  }
  
};

// -----------------------------------------------------------------------
// -----------------------------------------------------------------------

class ABI_EXPORT IE_MagickGraphic_Sniffer : public IE_ImpGraphicSniffer
{
public:
  IE_MagickGraphic_Sniffer () {}
  ~IE_MagickGraphic_Sniffer () {}
  
  virtual UT_Confidence_t recognizeContents (const char * szBuf, 
					     UT_uint32 iNumbytes)
  {
    // TODO: make me smarter
    return UT_CONFIDENCE_GOOD;
  }
  
  virtual UT_Confidence_t recognizeSuffix (const char * szSuffix)
  {
    // TODO: make me smarter
    return UT_CONFIDENCE_GOOD;
  }
  virtual bool getDlgLabels (const char ** szDesc,
			     const char ** szSuffixList,
			     IEGraphicFileType * ft)
  {
    *szDesc = "ImageMagick Graphics";
    *szSuffixList = "*.*";
    *ft = getType ();
    return true;
  }
  
  virtual UT_Error constructImporter (IE_ImpGraphic ** ppieg)
  {
    *ppieg = new IE_MagickGraphic ();
    if (*ppieg == 0)
      return UT_IE_NOMEMORY;
    return UT_OK;
  }  
};

// -----------------------------------------------------------------------
//
//      Abiword Plugin Interface 
//
// -----------------------------------------------------------------------

ABI_PLUGIN_DECLARE("ImageMagick")

IE_MagickGraphic_Sniffer *m_sniffer = 0;

ABI_FAR_CALL
int abi_plugin_register (XAP_ModuleInfo * mi)
{
    mi->name = "ImageMagick Plugin";
    mi->desc = "ImageMagick++ support for AbiWord. This will enable AbiWord to read any graphic type that ImageMagick supports.";
    mi->version = ABI_VERSION_STRING;
    mi->author = "Dom Lachowicz <cinamod@hotmail.com>";
    mi->usage = "No Usage";

    if (!m_sniffer)
      {
	// create a new sniffer
	m_sniffer = new IE_MagickGraphic_Sniffer ();
      }
    else
      {
	// else just reference the one we've got
	m_sniffer->ref();
      }

    // initialize ImageMagick and register ourself
    MagickLib::InitializeMagick(".");    
    IE_ImpGraphic::registerImporter ( m_sniffer );

    return 1;
}


ABI_FAR_CALL
int abi_plugin_unregister (XAP_ModuleInfo * mi)
{
    mi->name = 0;
    mi->desc = 0;
    mi->version = 0;
    mi->author = 0;
    mi->usage = 0;

    if ( m_sniffer )
      IE_ImpGraphic::unregisterImporter ( m_sniffer );

    if (!m_sniffer->unref())
      {
	m_sniffer = 0;
      }

    return 1;
}


ABI_FAR_CALL
int abi_plugin_supports_version (UT_uint32 major, UT_uint32 minor, UT_uint32 release)
{
  // true if >= 0.9.5
  if ( major >= 1 ) // >= 1.0 is true
    return 1;
  else if ( major == 0 && minor == 9 && release >= 5 )
    return 1;
  return 0;
}
